/****************************************************************************
**
** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** Licensees holding valid Qt Preview licenses may use this file in
** accordance with the Qt Preview License Agreement provided with the
** Software.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about Qt Commercial License Agreements.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef QWIDGET_QWS_P_H
#define QWIDGET_QWS_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of other Qt classes.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <qimage.h>
#include <qwsmemid_qws.h>
#include <private/qsharedmemory_p.h>

#ifndef QT_WINDOW_SURFACE

class QWSLock;

class QWSBackingStore
{
public:
    QWSBackingStore();
    ~QWSBackingStore();

    void create(QSize size, QImage::Format, int windowType);
    bool createIfNecessary(QWidget *tlw);
#ifndef QT_NO_QWS_MULTIPROCESS
    void attach(QWSMemId id, QSize size, QImage::Format, int windowType);
#endif
    void detach();
    void setMemory(QWSMemId id, const QSize &size, QImage::Format, int windowType);

    bool lock(int timeout = -1);
    void unlock();
    bool wait(int timeout = -1);
    void setLock(QWSLock *l);

    const QImage &image() const { return img; }
    QPaintDevice *paintDevice() { return &img; }

    void blit(const QRect &src, const QPoint &dest);

    QWSMemId memoryId() const;
    QSize size() const { return img.size(); }

    bool isNull() const { return img.isNull(); }

    enum WindowType {Transparent=0, Opaque=1, ReservedRegion=2, NonBuffered=3, DebugHighlighter=4, Invalid=5};

    int windowType() const { return _windowType; }

    QPoint tlwOffset() const { return offs; }
private:
    QImage img;
#ifndef QT_NO_QWS_MULTIPROCESS
    QSharedMemory shm;
#endif
    uchar *mem;

    QRegion clipRgn;
    QPoint offs;
    bool isSharedMemory : 1;
    bool ownsMemory : 1;

    int _windowType;
    QRegion windowRgn;

    QWSLock *memLock;
};

#endif // QT_WINDOW_SURFACE

#endif // QWIDGET_QWS_P_H
